'FFBSKAT' <- function(formula, phenodata, genodata, kin, nullmod, return.nullmod = FALSE, reg = NULL, sliding.window = c(20, 10), kernel = 'linear.weighted', beta.par = c(1, 25), weights = NULL, mode = 'add', method = 'Kuonen', acc = 1e-6, ncores = 1) {

if (!mode %in% c('add', 'dom', 'rec')) stop("Mode should be one of 'add', 'dom', 'rec'")

if (!kernel %in% c('linear.weighted', 'quadratic', 'IBS', 'IBS.weighted', '2wayIX')) stop("Mode should be one of 'linear.weighted', 'quadratic', 'IBS', 'IBS.weighted', '2wayIX'")

if (class(genodata) == 'snp.data') {
	typ <- 1
} else if (class(genodata) == 'gwaa.data') {
	genodata <- genodata@gtdata
	typ <- 1
} else if (class(genodata) %in% c('matrix', 'data.frame')) {
	typ <- 2
} else {
	stop("genodata should be of gwaa.data, matrix or data.frame class")
} 

if(!missing(phenodata)){
if (class(phenodata) == 'gwaa.data') {
	phenodata <- phenodata@phdata
} else if (!class(phenodata) %in% c('matrix', 'data.frame')) {
	stop("phenodata should be of gwaa.data or data.frame class")
}
if (dim(genodata)[1] != dim(phenodata)[1]) stop("Dimensions of phenodata and genodata do not match")
}

k <- dim(genodata)[2]

if (!is.null(reg)) {
	if (length(reg) != k) stop("Dimensions of reg and genodata do not match")
	l <- unique(reg)
	nreg <- length(l)
} else {
	if (k < sliding.window[1]) stop("Window size should not exceed the number of genetic variants")
	if (sliding.window[1] < 1) stop("Window size should be >= 1")
	if (sliding.window[2] < 1) stop("Step should be >= 1")
	nreg <- floor((k - sliding.window[1]) / sliding.window[2] + 1)
	p1 <- sapply(1:nreg, function(i) (i - 1)*sliding.window[2] + 1)
	p2 <- p1 + sliding.window[1] - 1
	l <- paste(p1, p2, sep = "_")
}

if (is.null(weights)) {
	fweights <- function(maf) ifelse(maf > 0, dbeta(maf, beta.par[1], beta.par[2]), 0)
} else {
	if (is.function(weights)) {		
		fweights <- weights
	} else {
		fweights <- NULL
		if (length(weights) != k) stop("Dimensions of weights and genodata do not match")
	}
}

if (missing(nullmod) & missing(formula)) stop("Either formula or nullmod should be given")

if (!missing(formula)) {
	if (is(try(formula, silent = TRUE), "try-error")) {
		formula <- phenodata[[as(match.call()[["formula"]], "character")]]
	} else {
	if (!is(try(as.formula(formula), silent = TRUE), "try-error")) {formula <- as.formula(formula)}
}}

run.null <- 0

if (!missing(nullmod)) {
	if (!missing(formula)) {
		if (is.null(nullmod$call$formula)) {
			warning("nullmod object has no 'call' info, running null model...")
			run.null <- 1
		} else {
			run.null <- 1
			if(!is(try(nullmod$call$formula, silent = TRUE), "try-error")&!is(try(formula, silent = TRUE), "try-error")){
				if (length(nullmod$call$formula ) == length(formula)) {
					if (nullmod$call$formula == formula) run.null <- 0
				}
			}
			if(!is(try(as.formula(formula), silent = TRUE), "try-error")&!is(try(as.formula(nullmod$call$formula), silent = TRUE), "try-error")){
				if (as.formula(formula) == as.formula(nullmod$call$formula)) run.null <- 0
			}
			if(!is(try(phenodata[,as.character(nullmod$call$formula)], silent = TRUE), "try-error")){
				nf <- phenodata[,as.character(nullmod$call$formula)]
				if (length(nf) == length(formula)) {
					if (all.equal(nf, formula))	run.null <- 0
				}
			}
			if(run.null) warning("nullmod object has different call, running null model...")
		}
	} else {
		if (!is.null(nullmod$call$formula)) {
			if (length(as.character(nullmod$call$formula))>1) {
				formula <- nullmod$call$formula
				cat("Formula from nullmod call is in effect\n")
			}
		}
	}
	if (!run.null) {
		if (is.null(nullmod$res) & is.null(nullmod$Inv)) {
			warning("nullmod object has no InvSigma and residualY data, running null model...")
			run.null <- 1
		}
	}
} else { run.null <- 1 }

if (run.null) {
	if (!missing(formula)) {
		if (missing(kin)) stop("Kinship matrix is missing")
		if (missing(phenodata)) stop("Phenotype data is missing")
		if (!missing(kin) & !missing(phenodata)) {
			phenodata <- as.data.frame(phenodata)
			if (!is(try(as.formula(formula), silent = TRUE), "try-error")){
				nullmod <- polygenic(as.formula(formula), data = phenodata, kinship.matrix = kin, quiet = TRUE)
				nullmod$call$formula <- as.formula(formula)
			} else {
			nullmod <- polygenic(formula, data = phenodata, kinship.matrix = kin, quiet = TRUE)
			nullmod$call$formula <- formula
			}
		}
	} else { stop("No formula to run null model") } 
} else {
if(!missing(phenodata)) {
	if (dim(phenodata)[1] != length(nullmod$res)) stop("Dimensions of nullmod phenotype and covariates do not match")
}
	if (dim(genodata)[1] != length(nullmod$res)) stop("Dimensions of nullmod phenotype and genodata do not match")
}

SIGMAi <- nullmod$Inv
res <- nullmod$res

if (dim(SIGMAi)[1] != length(res)) {
if(!is.null(rownames(SIGMAi))& !is.null(rownames(genodata))& !is.null(names(nullmod$res))){
	vec <- match(rownames(SIGMAi), names(nullmod$res))
	res <- nullmod$res[vec]
	vec <- match(rownames(SIGMAi), rownames(genodata))
	genodata <- genodata[vec, ]
} else {
if (is(try(nullmod$measuredIDs, silent = TRUE), "try-error")) stop("Dimensions of nullmod residuals and InvSigma do not match")
res <- nullmod$res[nullmod$measuredIDs]
genodata <- genodata[nullmod$measuredIDs, ]
}
}
n <- length(res)

X <- matrix(rep(1, n), n, 1)

if (!missing(formula)) {
	if (!is(try(as.formula(formula), silent = TRUE), "try-error")){
		if (is(try(covariates <- as.matrix(model.frame(formula, phenodata)), silent = TRUE), "try-error"))stop("Check whether all covariates are given in phenodata")
		covariates <- as.matrix(model.frame(formula, phenodata, na.action = na.omit, drop.unused.levels = TRUE)[, -1])
		if (dim(covariates)[1] != n) stop("Dimensions of nullmod phenotype and covariates do not match")
		X <- as.matrix(cbind(matrix(rep(1, n), n, 1), covariates))
	}
}

#cat("Total sample size after omitting NA's:", n, "individuals\n")

if (ncores < 1)ncores <- 1
if (ncores > 1) {
	ncores <- min(ncores,nreg)
	if (ncores > detectCores()) {
		ncores <- detectCores()
		warning("Number of CPUs available: ", ncores)
	}
}

SIG_res <- crossprod(SIGMAi, res)					#SIG_res=SIGMAi%*%res
SiX <- crossprod(SIGMAi, X)							#SiX=SIGMAi%*%X

if (kernel == 'linear.weighted') {
	SPS <- SIGMAi - SiX %*% solve(t(X) %*% SiX, t(SiX)) 
}
if (kernel != 'linear.weighted') {
	P11 = diag(dim(SIGMAi)[1]) - X %*% solve(t(X) %*% SiX, t(SiX))
	CholSigmaiP11 = chol(SIGMAi) %*% P11
}

analyse <- function(region) {
	if (typ==1) Z <- as.numeric(genodata[, region])
	if (typ==2) Z <- genodata[, region]
	Z <- as.matrix(Z)
	m0 <- dim(Z)[2]
	v <- sapply(1:dim(Z)[2], function(x) var(Z[, x], na.rm = TRUE))
	Z <- as.matrix(Z[, v > 0])
	if (dim(Z)[2] == 0) {
		warning("No polymorphic variants in region ", r, ", skipped")
		return(c('NA',m0,0))
	}

	MAF <- colMeans(Z, na.rm = TRUE) / 2

	if (mode == 'dom') {
		Z <- round(Z)
		Z[Z == 2] <- 1
	}
	else if (mode == 'rec') {
		Z <- round(Z)
		Z[Z == 1] <- 0
		Z[Z == 2] <- 1
	}
		  
	#impute missing genotypes
	for (z in 1:dim(Z)[2]) Z[is.na(Z[, z]), z] <- mean(Z[, z], na.rm = T)

	if (any(MAF > .5)) warning("Check whether minor allele homozygotes are coded as 2 at region ", r)

	# detect and eliminate linear-dependent genetic variants
	dqr = qr(Z)										# QR-decomposition
	index = dqr$pivot[1:dqr$rank]					# indexes of dependent genetic variants
	Z = as.matrix(Z[,index])						# elimination
	MAF <- MAF[index]

	if (!is.null(fweights)) {
		w <- fweights(MAF)
	} else {
		w <- weights[region]
	}

	if (kernel != 'linear.weighted') {
	n = nrow(Z)
	m = ncol(Z) 
	K = lskmTest.GetKernel(Z, kernel, w, n, m)
	Q = sum(SIG_res * crossprod(K, SIG_res))
	KKK = CholSigmaiP11 %*% K %*% t(CholSigmaiP11)
	}

	if (kernel == 'linear.weighted') {
	Q05 <- w * crossprod(Z, SIG_res)				# W %*% t(G) %*% Sigmai %*% res
	Q <- sum(Q05 * Q05)
	KerMat <- t(Z) %*% SPS %*% Z
	KKK <- t(KerMat * w) * w
	}

	eig <- eigen(KKK, symmetric = TRUE, only.values = TRUE)
	ev <- eig$values[eig$values > 1e-6*eig$values[1]]

	if (method == 'Kuonen') p <- pchisqsum(Q, rep(1, length(ev)), ev, lower.tail = F, method = 'sad')
	if (method == 'Davies')	p <- davies(Q, ev, acc = acc)$Qq
	p <- max(min(p, 1), 0)
	c(p,m0,dim(Z)[2])

}

if (ncores == 1) {

	out <- data.frame(region = l, pvalue = rep(NA, nreg), markers = rep(NA, nreg), polymorphic.markers = rep(NA, nreg))

	for (i in 1:nreg) {
		r <- as.character(l[i])
		if (!is.null(reg)) {region = reg == l[i]} else {region = p1[i]:p2[i]}
		out[i,2:4] <- analyse(region)
	}

} else {

	nj <- floor(nreg / ncores)
	vj <- rep(nj, ncores)
	nadd <- nreg %% ncores
	if (nadd > 0) vj[1:nadd] <- nj + 1

	cl <- makeCluster(ncores, type = 'SOCK')
	registerDoSNOW(cl)

	out <- foreach(j = 1:ncores, .combine = rbind, .inorder = F) %dopar% {

	if (typ == 1) library(GenABEL)

	if (method == 'Davies') {
		library(CompQuadForm)
	} else if (method == 'Kuonen') {
		library(survey)
	}

	if (kernel != 'linear.weighted') library(SKAT)
	
	i0 <- (sum(vj[1:(j-1)]) * (j > 1) + 1)
	out <- c()

	for (i in i0:(i0 + vj[j] - 1)) {
		r <- as.character(l[i])
		if (!is.null(reg)) {region = reg == l[i]} else {region = p1[i]:p2[i]}
		out <- rbind(out, c(r, analyse(region)))
	}
	out
	}

	stopCluster(cl)
	out <- as.data.frame(out)
	out[, 2:4] <- sapply(2:4, function(x) as.numeric(as.character(out[, x])))
	colnames(out) <- c('region', 'pvalue', 'markers', 'polymorphic.markers')
	out <- out[match(l, out$r), ]

}

if (is.null(reg)) {
	if (p2[nreg] < k) {
		out <- rbind(out, rep(NA))
		i <- nreg + 1
		p1 <- (i - 1) * sliding.window[2] + 1
		p2 <- k
		r <- paste(p1, p2, sep = "_")
		levels(out$region) <- c(levels(out$region), r)
		out[i,1] <- r
		out[i,2:4] <- analyse(p1:p2)
	}
}

out0 <- out
out <- c()
out$results <- out0
out$sample.size <- n
if (return.nullmod) out$nullmod <- nullmod

out
}
