\name{FFBSKAT}
\alias{FFBSKAT}
\title{Fast Family-Based SKAT}
\description{
  A regional association analysis in related samples	
}
\usage{
FFBSKAT(formula, phenodata, genodata, kin, nullmod,
return.nullmod = FALSE, reg = NULL, sliding.window = c(20, 10),
kernel = "linear.weighted", beta.par = c(1, 25), weights = NULL,
mode = "add", method = "Kuonen", acc = 1e-6, ncores = 1)
}
\arguments{
	\item{formula}{referring to the column(s) in \code{phenodata} to be analysed as outcome and,
	if needed, covariates.}
	
	\item{phenodata}{a data frame containing columns mentioned in \code{formula}: trait to analyse and,
	if needed, covariates.}
	
	\item{genodata}{an object of \code{\link{gwaa.data-class}} or \code{\link{snp.data-class}}. 
	It can be also a data frame or matrix (with ids given in rows) containing genotypes 
	coded as AA = 0, Aa = 1 and aa = 2, where a is a minor allele. NAs in \code{genodata}
	will be imputed by the mean values. Monomorphic and duplicated variants will be omitted.}
	
	\item{kin}{a kinship matrix to evaluate the null model. It can be calculated from a pedigree
	(e.g. using \code{kinship2} package) or from genotypic data (with \code{\link{ibs}} function of GenABEL).}

	\item{nullmod}{an object of \code{\link{polygenic}} data class. Setting \code{nullmod} allows
	to avoid re-estimation of the null model that does not depend on genotypes and can be
	calculated once for a trait.
	\code{nullmod} object in proper format can be obtained by running either \code{FFBSKAT} with \code{return.nullmod = TRUE}
	or \code{\link{polygenic}} function. Entries used in \code{nullmod} are \code{$residualY} and \code{$InvSigma}
	with non-empty names. \code{nullmod$call$formula} will be compared to the\cr
	\code{formula} given in input. Note that, if covariates are mentioned in \code{formula} or
	\code{nullmod$call$formula}, they should be given in \code{phenodata} to be used in kernel matrix calculation.}

	\item{return.nullmod}{logical value indicating whether a \code{nullmod} object should be returned.}	

	\item{reg}{a vector assigning a region for each genetic marker in \code{genodata}.
	If NULL, \code{sliding.window} parameters are used.}

	\item{sliding.window}{the sliding window size and step. Has no effect if \code{reg} is defined.}

	\item{kernel}{one of "linear.weighted" (default), "quadratic", "IBS", "IBS.weighted", "2wayIX".
	See Details for "linear.weighted" kernel description and [Wu, 2011] for other kernel types.
	"2wayIX" kernel considers SNP-SNP interaction terms along with main effects. For
	"linear.weighted" and "IBS.weighted" kernels, weights can be varied by defining
	\code{weights} or \code{beta.par}.}

	\item{beta.par}{two positive numeric shape parameters in the beta distribution to assign weights 
	for each SNP in weighted kernels (see Details). Default = c(1, 25) is recommended for analysis
	of rare variants. Has no effect for unweighted kernels or if \code{weights} are defined.}

	\item{weights}{a numeric vector or a function of MAF to assign weights for each SNP in the
	weighted kernels. Has no effect if one of unweighted kernels was chosen. If NULL, the weights will
	be calculated using beta distribution (see Details)}

	\item{mode}{the mode of inheritance: "add", "dom" or "rec" for additive, dominant or recessive mode,
	respectively. For dominant (recessive) mode genotypes will be recoded as AA = 0, Aa = 1 and aa = 1
	(AA = 0, Aa = 0 and aa = 1), where a is a minor allele. Default mode is additive.}
	
	\item{method}{either "Kuonen" or "Davies". Method of computing the p-value. Default = "Kuonen".}

	\item{acc}{accuracy parameter for "Davies" method.}
	
	\item{ncores}{number of CPUs for parallel calculations. Default = 1}
}
\details{
	By default, FFBSKAT uses the linear weighted kernel function, \eqn{K = GWWG^T}, 
	where \eqn{\mathit{G}} is the \eqn{\mathit{n\times p}} genotype matrix for \eqn{\mathit{n}} individuals and \eqn{\mathit{p}} SNPs in the region, 
	and \eqn{\mathit{W}} is the \eqn{\mathit{p\times p}} diagonal weight matrix. Given shape parameters of the beta function, \code{beta.par = c(a, b)}, 
	the weights are defined using probability density function of the beta distribution:\cr
	\cr
	\eqn{W_{i}=(B(a,b))^{^{-1}}MAF_{i}^{a-1}(1-MAF_{i})^{b-1} },\cr
	\cr
	where \eqn{MAF_{i}} is a minor allelic frequency for \eqn{i^{th}} SNP in region, which is estimated from genotypes, and \eqn{B(a,b)} is a beta function. This way of defining weights is the same as in original SKAT (see [Wu, 2011] for details).
	The formula: \cr
	\cr
	\eqn{Q=0.5\tilde{y}^{T}\Omega^{-1}K\Omega^{-1}\tilde{y}}\cr
	\cr
	is used to calculate score statistic, where \eqn{\tilde{y}} and \eqn{\Omega} are environmental residuals and covariance matrix obtained under the null hypothesis, respectively. Depending on the method option chosen, either Kuonen or Davies method is used to calculate p-values from the score statistic Q. Both an Applied Statistics algorithm that inverts the characteristic function of the mixture chisq [Davies, 1980] and a saddlepoint approximation [Kuonen, 1999] are nearly exact, with the latter usually being a bit faster.
	For other kernel types, see [Wu, 2011].
}
\value{
	A list with values:

	\item{results}{a data frame containing p-values, numbers of variants and informative polymorphic variants for each of analyzed regions.}	

	\item{sample.size}{after omitting NA's in trait and, if used, covariates.}	

	\item{nullmod}{a null model object of class \code{polygenic}, returned if\cr
	\code{return.nullmod = TRUE}.}
}
\references{
	Davies R.B. (1980) Algorithm AS 155: The Distribution of a Linear Combination of chi-2 Random Variables, Journal of the Royal Statistical Society. Series C (Applied Statistics), Vol. 29, N 3, P. 323-333.\cr
	Kuonen D. (1999) Saddlepoint Approximations for Distributions of Quadratic Forms in Normal Variables. Biometrika, Vol. 86, No. 4, P. 929-935.\cr
	Wu M.C. et al. (2011) Rare-variant association testing for sequencing data with the sequence kernel association test. Am. J. Hum. Genet., Vol. 89, P. 82-93.
}
\author{
	Gulnara Svishcheva \email{gulsvi@mail.ru}\cr
	Nadezhda Belonogova \email{belon@bionet.nsc.ru}
}
\seealso{
    \code{\link{FFBSKAT-package}}\cr
    \code{\link{GenABEL}}
}
\examples{

library(FFBSKAT)
data(example.data)  

## Run FFBSKAT with sliding window (default):
out1 <- FFBSKAT(trait ~ age + sex, phenodata, genodata, kin,
	return.nullmod = TRUE)

## Run FFBSKAT with regions defined in snpdata$gene and with
## null model obtained in first run:
out2 <- FFBSKAT(nullmod = out1$nullmod, phenodata = phenodata,
	genodata = genodata, reg = snpdata$gene)

}
